<?php

/**
 * @author          Tassos Marinos <info@tassos.gr>
 * @link            http://www.tassos.gr
 * @copyright       Copyright © 2021 Tassos Marinos All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 */

namespace NRFramework\Helpers\Widgets;

defined('_JEXEC') or die;

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.path');

use NRFramework\File;
use NRFramework\Mimes;
use NRFramework\Image;

class GalleryManager
{
	/**
	 * The file prefix when file is first uploaded to the tmp folder
	 * 
	 * @var  string
	 */
	const tmp_file_prefix = 'nrf_gallery_';
	
	/**
	 * Upload file
	 *
	 * @param	array	$file						The request file as posted by form
	 * @param	string	$upload_settings			The upload settings
	 * @param	array	$media_uploader_file_data	Media uploader related file settings
	 * @param   array   $resizeSettings				The resize settings
	 *
	 * @return	mixed	String on success, Null on failure
	 */
	public static function upload($file, $upload_settings, $media_uploader_file_data, $resizeSettings)
	{
		$ds = DIRECTORY_SEPARATOR;
		
		$upload_folder = rtrim(ltrim($upload_settings['upload_folder'], $ds), $ds);
		$base_upload_directory = JPATH_ROOT . $ds . $upload_folder;

		// Create destination folder if missing
		if (!$media_uploader_file_data['is_media_uploader_file'] && !File::createDirs($base_upload_directory))
		{
			return false;
		}

		// The source file name
		$source = '';

		/**
		 * We transfer the image to the tmp folder when:
		 * - The image came from the Joomla Media Manager and we enabled to copy it over to the Gallery Root Folder
		 * - The image was uploaded manually
		 */
		if (($media_uploader_file_data['is_media_uploader_file'] && $upload_settings['copy_existing_images']) || !$media_uploader_file_data['is_media_uploader_file'])
		{
			try {
				$source = File::upload($file, null, $upload_settings['allowed_types'], $upload_settings['allow_unsafe'], self::tmp_file_prefix);
			} catch (\Throwable $th)
			{
				return false;
			}
		}

		// If the file came from the Media Manager file and we are copying it, fix its filename
		if ($media_uploader_file_data['is_media_uploader_file'] && isset($media_uploader_file_data['copied_from_media_uploader']) && !$media_uploader_file_data['copied_from_media_uploader'])
		{
			$media_uploader_file_data['media_uploader_filename'] = self::getFilePathFromMediaUploaderFile($media_uploader_file_data['media_uploader_filename']);
		}

		// The uploaded file name
		$filename = null;

		// whether to copy the base image
		$copyBaseImage = true;

		// if the file came from the Media Manager and we are not copying the file over to the `upload_folder`
		if ($media_uploader_file_data['is_media_uploader_file'] && !$upload_settings['copy_existing_images'])
		{
			$copyBaseImage = false;
			$source = JPATH_ROOT . $ds . ltrim($media_uploader_file_data['media_uploader_filename'], $ds);
			$filename = $media_uploader_file_data['media_uploader_filename'];
		}
		
		// Set destination
		$destination = $base_upload_directory . $ds . str_replace(self::tmp_file_prefix, '', basename($source));
		
		if ($copyBaseImage && !$source = File::move($source, $destination, $upload_folder, $upload_settings['allow_unsafe']))
		{
			return false;
		}

		// generate thumbnails
		if (!$thumb_data = self::generateThumbnail($source, $destination, $resizeSettings))
		{
			return false;
		}

		// Check whether to copy and resize the original image
		if ($copyBaseImage && $resizeSettings['original_image_resize'])
		{
			Image::resizeAndKeepAspectRatio($destination, $resizeSettings['original_image_resize_width'], $resizeSettings['original_image_resize_quality']);
		}

		return [
			'filename' => is_null($filename) ? $thumb_data['filename'] : $filename,
			'thumbnail' => $thumb_data['resized_filename']
		];
	}

	/**
	 * Media Uploader files look like: https://example.com/images/sampledata/parks/banner_cradle.png
	 * We remove the first part (https://example.com/images/) and keep the other part (relative path to image).
	 * 
	 * @param   string  $filename
	 * 
	 * @return  string
	 */
	private static function getFilePathFromMediaUploaderFile($filename)
	{
		$filenameArray = explode('images/', $filename, 2);
		unset($filenameArray[0]);
		$new_filepath = join($filenameArray);
		return 'images/' . $new_filepath;
	}

	/**
	 * Generates thumbnail
	 * 
	 * @param   string  $source			 Source image path.
	 * @param   string  $destination	 Destination image path (without _thumb).
	 * @param   array   $resizeSettings	 Resize Settings.
	 * 
	 * @return  array
	 */
	public static function generateThumbnail($source, $destination, $resizeSettings)
	{
		// Generate destinatino, thumbnail path
		$parts = pathinfo($destination);
		$destination = $parts['dirname'] . DIRECTORY_SEPARATOR . $parts['filename'] . '_thumb.' . $parts['extension'];

		$resized_image = $resizeSettings['style'] == 'masonry'
						 ?
						 Image::resizeAndKeepAspectRatio($source, $resizeSettings['thumb_width'], $resizeSettings['resize_quality'], $destination)
						 :
						 Image::resize($source, $resizeSettings['thumb_width'], $resizeSettings['thumb_height'], $resizeSettings['resize_quality'], $resizeSettings['resize_method'], $destination);
		
		if (!$resized_image)
		{
			return;
		}

		return [
			'filename' => basename($source),
			'resized_filename' => basename($resized_image)
		];
	}

	/**
	 * Delete an uploaded file
	 *
	 * @param  string  $filename		The filename
	 * @param  string  $filename		The thumbnail filename
	 * @param  string  $upload_folder   The uploaded folder
	 * @param  array   $settings		Other settings we need for the deletion
	 *
	 * @return bool
	 */
	public static function deleteFile($filename, $thumbnail, $upload_folder, $settings)
	{
		if (empty($filename) || empty($upload_folder))
		{
			return false;
		}

		$is_media_uploader_file = $settings['is_media_uploader_file'];
		$copied_from_media_uploader = $settings['copied_from_media_uploader'];

		$file = \JPath::clean(JPATH_ROOT . DIRECTORY_SEPARATOR . $upload_folder . DIRECTORY_SEPARATOR . $filename);

		$deleted_file = '';

		// delete the main image if its not added by the Media Uploader
		if ((!$is_media_uploader_file || ($is_media_uploader_file && $copied_from_media_uploader)) && \JFile::exists($file))
		{
			$deleted_file = \JFile::delete($file);
		}

		// check if we have a thumbnail and delete it
		$file = \JPath::clean(JPATH_ROOT . DIRECTORY_SEPARATOR . $upload_folder . DIRECTORY_SEPARATOR . $thumbnail);

		if (!\JFile::exists($file))
		{
			return $deleted_file;
		}

		return \JFile::delete($file);
	}
}